package org.jboss.test.xml.resolverwithqnamemapping.support;

public class Base
{

   private String data;

   public Base()
   {
      super();
   }

   public String getData()
   {
      return data;
   }

   public void setData(String data)
   {
      this.data = data;
   }

}