/*
  * JBoss, Home of Professional Open Source
  * Copyright 2005, JBoss Inc., and individual contributors as indicated
  * by the @authors tag. See the copyright.txt in the distribution for a
  * full listing of individual contributors.
  *
  * This is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License as
  * published by the Free Software Foundation; either version 2.1 of
  * the License, or (at your option) any later version.
  *
  * This software is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this software; if not, write to the Free
  * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
  * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
  */
package org.jboss.test.xml;

import java.net.URL;

import org.jboss.xb.binding.Unmarshaller;
import org.jboss.xb.binding.UnmarshallerFactory;
import org.jboss.xb.binding.sunday.unmarshalling.SchemaBinding;
import org.jboss.xb.binding.sunday.unmarshalling.XsdBinder;
import org.jboss.test.xml.po.PurchaseOrder;
import org.jboss.test.BaseTestCase;

/**
 * @author <a href="mailto:alex@jboss.org">Alexey Loubyansky</a>
 * @version <tt>$Revision: 45813 $</tt>
 */
public class PoUnitTestCase
   extends BaseTestCase
{
   public PoUnitTestCase(String localName)
   {
      super(localName);
   }

   public void testMain() throws Exception
   {
      String xsdName = "xml/po.xsd";
      URL xsd = getURL(xsdName);
      String xmlName = "xml/po.xml";
      URL xml = getURL(xmlName);

      SchemaBinding schema = XsdBinder.bind(xsd.toExternalForm());
      Unmarshaller unmarshaller = UnmarshallerFactory.newInstance().newUnmarshaller();
      PurchaseOrder po = (PurchaseOrder)unmarshaller.unmarshal(xml.toExternalForm(), schema);
      assertEquals(PurchaseOrder.DEFAULT_INSTANCE, po);
   }

   private URL getURL(String name)
   {
      URL resource = Thread.currentThread().getContextClassLoader().getResource(name);
      if(resource == null)
      {
         fail("resource not found: " + name);
      }
      return resource;
   }
}
