/*
* JBoss, Home of Professional Open Source
* Copyright 2006, JBoss Inc., and individual contributors as indicated
* by the @authors tag. See the copyright.txt in the distribution for a
* full listing of individual contributors.
*
* This is free software; you can redistribute it and/or modify it
* under the terms of the GNU Lesser General Public License as
* published by the Free Software Foundation; either version 2.1 of
* the License, or (at your option) any later version.
*
* This software is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this software; if not, write to the Free
* Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
* 02110-1301 USA, or see the FSF site: http://www.fsf.org.
*/
package org.jboss.test.xb.builder.object.type.xmlenum.test;

import junit.framework.Test;
import org.jboss.test.xb.builder.object.type.xmlenum.support.ignorecase.Root;
import org.jboss.xb.builder.runtime.EnumValueAdapter;

/**
 * Ignore test case enum.
 *
 * @author <a href="mailto:ales.justin@jboss.com">Ales Justin</a>
 */
public class IgnoreCaseUnitTestCase extends AbstractDefaultsTest<Root>
{
   public IgnoreCaseUnitTestCase(String name)
   {
      super(name, Root.class);
   }

   public static Test suite()
   {
      return suite(IgnoreCaseUnitTestCase.class);
   }

   protected void testEnumValueAdapter(EnumValueAdapter enumValueAdapter)
   {
      super.testEnumValueAdapter(enumValueAdapter);

      assertEquals(Root.ONE, enumValueAdapter.cast("one", Root.class));
      assertEquals(Root.ONE, enumValueAdapter.cast("One", Root.class));
      assertEquals(Root.ONE, enumValueAdapter.cast("ONE", Root.class));

      assertEquals(Root.TWO, enumValueAdapter.cast("two", Root.class));
      assertEquals(Root.TWO, enumValueAdapter.cast("Two", Root.class));
      assertEquals(Root.TWO, enumValueAdapter.cast("TWO", Root.class));

      assertEquals(Root.THREE, enumValueAdapter.cast("three", Root.class));
      assertEquals(Root.THREE, enumValueAdapter.cast("Three", Root.class));
      assertEquals(Root.THREE, enumValueAdapter.cast("THREE", Root.class));      
   }
}
